function [out, Ct, Pt, Dt, Qt, Gt, U] = a_ll(xx, T_f, SET, data, mats_only)
%function out = ll(params, SET, data)
%
% Calculates the log-likelihood for Gaussian sequence given a mean and
% variance sequence.
%
% State space representation:
% 
% S_t = C(t) + P(t) S_{t-1} + D(t) n_t : State-state equation
% X_t = H S_t + v_t : observation equation

SET=SET.a ;

n_   = SET.variable.n_ ; 
nobs = SET.nobs ;

if nargin<6 ; mats_only=0 ; end

%% Re-solve model with xx parameters

SET.M_.params(SET.params_to_estimate_idx) = xx ;

if sum(SET.M_.params(SET.params_to_estimate_idx)<0)>0
    out=-1e12;
    %disp('out of bounds') ;
    return
end

dyn_in.M_  = SET.M_ ;
dyn_in.options_ = SET.options_ ;
dyn_in.oo_  = SET.oo_ ;
dyn_in.solve = 1 ;
dyn_in.linearize_around_diff_y = 0 ;

try
    mats = a_resolve_(SET,dyn_in) ;
catch
    out = -10e12 ;
    return
end
    
if isempty(mats.mats.Q)
    out=-1e12;
    disp('not unq') ;
    return
end

SET.mat_i_f_zlb = mats.mat_i_f_zlb ;
SET.mat_init = mats.mat_init ;
SET.mats.Q = mats.mats.Q ;

Qt = repmat(mats.mats.Q,[1,1,SET.ss]) ;
Gt = repmat(mats.mats.G,[1,1,SET.ss]) ;

if max(T_f)>0 
    mats = a_rfmats(SET, 1, max(T_f)) ;
    for ii=1:SET.ss
        if T_f(ii)>0
            Qt(:,:,ii) = mats.Qhat(:,:,max(T_f)-T_f(ii)+1) ;
            Gt(:,:,ii) = mats.Ghat(:,:,max(T_f)-T_f(ii)+1) ;
        end
    end
end
    
%% Extract matrices for Kalman filter

Ct = Qt(1:n_-1,end,:) ;
Pt = Qt(1:n_-1,1:n_-1,:) ;
Dt = Gt(1:n_-1,:,:) ;

H = zeros(nobs,n_-1) ; 

for i=1:nobs
    H(i,SET.EST.obs_(i)) = 1 ;
end

if mats_only ; out=NaN; return ; end

Omega  = SET.Omega ;
DOmega = Dt(:,:,1)*Omega*Dt(:,:,1)' ;
SET.Sigma = dlyap_doubling(Pt(:,:,1),DOmega);

T_f_kf = ones(1,SET.ss) ; T_f_kf(T_f>0) = 0 ;

[U, St, flag] = kf_(SET, Ct, Pt, Dt, H, data, T_f_kf) ;

if flag ; out=-1e12 ; return ; end

out = 0;

for t=1:SET.ss
    hidx = SET.EST.hidx ;
    if T_f(t)>0 ; hidx = SET.EST.hidx_no_obs ; end
    
    out = out - (nobs/2) * log(2*pi) - ...
        (1/2) * log(det(St(hidx,hidx,t))) - ...
        (1/2) * U(hidx,t)' / St(hidx,hidx,t) * U(hidx,t) ;
end

if abs(imag(out))>0
    out=-1e12;
    disp('imag ll') ;
end
