
clc

%%

clearvars -except SET

%%

model_name = 'aggregate' ;

experiment = SET.experiment ;

%%

hydata    = 2.52*4;                                % U.S. stock of housing to income
dhdata    = 0.29; 
dydata    = dhdata*hydata;

sigmac    = 1;                                     % CRRA coefficient  
nu        = 2;                                     % labor supply elasticity
etan      = 1;                                     % choice of units

hbar      = 1;

rhohMA = eps;
rhomMA = eps;
rhor   = eps;

alphar    = 0.858;                                 % Primiceri et. al
alphap    = 1.709; 
alphax    = 0.208;
alphay    = 0.051; 

r = 0.02/4;                                % choose discount factor to match this interest rate

switch experiment
    
    case 'baseline'

        alph    = 3.44; 

        gamma   = 0.985; 
        mbar    = (1-gamma)*dhdata; 

        rho     = 0.96;
        rhow    = 0.86;
        psi     = 21;

        rhoz    = 0.96;
        rhom    = 0.50;
        rhoh    = 0.96;
        rhon    = 0.10;
        rhob    = 0.93;
        rhop    = 0.65;

        sigmaz  = 0.65;
        sigmam  = 0.94;
        sigmah  = 0.19;
        sigman  = 0.16;
        sigmab  = 1.22;
        sigmap  = 0.76;
        sigmar  = 0.80; 
        
end

% Compute Steady State

euler     = @(xchat, dydata, alph) 1 + dydata - xchat*(alph/(alph - 1) - 1/(alph - 1)*xchat^(1-alph))^(-1);

xchatss   = fzero(@(xchat) euler(xchat, dydata, alph), 7);
Delta     = sigmac/(alph - sigmac)*xchatss^(-alph);
beta      = ((1 + r)*(1+Delta))^(-1);

rss       = r; 
qss       = 1/(1 + r - gamma); 
Fss       = (alph/(alph-1) - 1/(alph-1)*xchatss.^(1-alph));

muss      = (etan)^(sigmac/(sigmac + nu)).*Fss.^(nu*sigmac/(sigmac + nu))./(beta.*(1 + r)).^(nu/(sigmac + nu)); 
lambdass  =  muss.*(1 - beta*(1 + r))./(1 - beta*gamma); 
chatss    = (beta.*(1 + r).*muss).^(-1/sigmac); 
css       = (muss./etan).^(1/nu);
xss       = chatss.*xchatss; 
yss       = css;
nss       = yss;
wss       = etan*nss^nu/muss;

phss      = hydata.*css; 
etah      = phss./beta.*muss.*(1 - beta - lambdass./muss.*mbar);

qbss      = mbar*phss./(1 - gamma); 
qass      = (xss - css); 

pss       = phss/hbar;
bss       = qbss./qss; 
ass       = qass./qss;
dyss      = qss*bss/css;

% Finish computing steady state

fprintf('\n')
fprintf('\n')
fprintf('Fraction Constrained:      Model                = %9.2f \n',       xchatss^(-alph));
fprintf('Implied discount factor                         = %9.3f \n',       beta);
fprintf('Implied weight on housing, etah                 = %9.3f \n',       etah);
fprintf('Gap rho - r annualized, *100                    = %9.3f \n',       (1./beta - 1 - r)*4*100);
fprintf('\n')

fprintf('Equilibrium r:             Model, Data          = %9.2f %9.2f\n',  [r*4*100,             2.00]);
fprintf('Housing to Income:         Model, Data          = %9.2f %9.2f\n',  [pss*hbar/css,        hydata]);
fprintf('Debt to Housing:           Model, Data          = %9.2f %9.2f\n',  [qss*bss/(pss*hbar),  dhdata]);
fprintf('Debt to Income:            Model, Data          = %9.2f %9.2f\n',  [qss*bss/css,         dhdata*hydata]);
fprintf('\n')

iss         = rss; 
paiss       = 1; 

params      = [beta; gamma; nu; etah; hbar; mbar; alph; etan; sigmac; rho; yss; rss; alphay; alphap; log(wss); log(muss); log(nss); rhow; psi; rhom; sigmam; rhoh; sigmah; rhoz; sigmaz; rhon; sigman; rhob; sigmab; alphar; alphax; rhor; sigmar; rhop; sigmap ; dyss ; log(paiss); log(pss) ; log(dyss); rhomMA ; rhohMA];     

xsteady     = [pss; qss; muss; hbar; lambdass; xss; css; chatss;  ass; bss; wss; nss; yss];
xsteady     = log(xsteady); 

save input params xsteady

%%

eval(['dynare ', model_name, '.mod noclearall nowarn']) ;

eval(['delete ', model_name, '.log']) ;
eval(['delete ', model_name, '_results.mat']) ;

dyn_in.M_  = M_ ;
dyn_in.oo_ = oo_ ;
dyn_in.solve = 1 ;
dyn_in.linearize_around_diff_y = 0 ;
dyn_in.options_ = options_ ;

SET.M_ = dyn_in.M_ ;
SET.oo_ = dyn_in.oo_ ;
SET.options_ = options_ ;

out = dyn_to_str(dyn_in) ;

SET.mats.Q = out.mats.Q ; Q = SET.mats.Q ;
SET.mats.G = out.mats.G ; G = SET.mats.G ;

SET.str_mats.A0 = out.mats.A ;
SET.str_mats.A1 = out.mats.B ;
SET.str_mats.B0 = out.mats.D ;
SET.str_mats.D0 = out.mats.E ;
SET.str_mats.D2 = out.mats.D2 ;
SET.str_mats.Gamma = out.mats.Gamma ;

%% Extract variable details from Dynare output

n_ = M_.endo_nbr ; % Number of endogenous variables
l_ = M_.exo_nbr ;  % Number of exogenous variables
n_ = n_+1 ;        % Constant

SET.variable.n_ = n_ ; 
SET.variable.l_ = l_ ; 
SET.nparam      = M_.param_nbr ;

var_names   = cellstr(M_.endo_names) ;
param_names = cellstr(M_.param_names) ;
exo_var_names = cellstr(M_.exo_names) ;

for ii=1:n_-1
    eval(['SET.variable.',var_names{ii},'=',int2str(ii),';']) ;
end

for ii=1:l_
    eval(['SET.variable.shock.',exo_var_names{ii},'=',int2str(ii),';']) ;
end

for ii=1:M_.param_nbr
    eval(['SET.param_names.',param_names{ii},'=',int2str(ii),';']) ;
end

%% Compute steady-state

SET.y_ss = (eye(n_-1) - Q(1:n_-1,1:n_-1)) \ Q(1:n_-1,end)  ;
SET.y_ss = [SET.y_ss ; 1] ; % Constant

%% ZLB algorithm parameters

SET.pos_of_i = SET.variable.iobs ;              % Position of interest rate in mod file variable declaration
SET.tr_row   = 17 ;                          % Row (line number) of Taylor rule in mod file
SET.maxchk   = 120 ;                          % Check ZLB binding out 40 periods
SET.stoch    = 1 ;                           % Shocks each period? For IRF, identical to 0
SET.zlb_val  = 0;%-M_.params(12)-0.02/4;        % Value of ihat at ZLB

% Structural matrices at ZLB
SET.mat_i_f_zlb = SET.str_mats ;

SET.mat_i_f_zlb.A0(SET.tr_row,:) = 0 ;
SET.mat_i_f_zlb.A1(SET.tr_row,:) = 0 ;
SET.mat_i_f_zlb.D0(SET.tr_row,:) = 0 ;
SET.mat_i_f_zlb.B0(SET.tr_row,:) = 0 ;
SET.mat_i_f_zlb.A0(SET.tr_row,SET.pos_of_i) = 1 ;
SET.mat_i_f_zlb.A0(SET.tr_row,end) = -SET.zlb_val ; % Constant

SET.mat_init    = SET.str_mats ; % Structural matrices not at ZLB
SET.mat_fin     = SET.str_mats ; % Structural matrices after ZLB

SET.zlb.Qf          = SET.mats.Q ; 
SET.zlb.mat_init    = SET.mat_init ; 
SET.zlb.mat_i_f_zlb = SET.mat_i_f_zlb ;

%% 

SET.a.Omega = eye(l_) ;

dyn_in.solve    = 0 ;
SET.tv_str_chg.flag = 0 ; 