%
% Aggregate Counterfactuals
% 

clear all
clc

warning('Off','all') ;

path(pathdef)

addpath('./output') ;
addpath('./aggregate') ;
addpath('./regional') ;
addpath('./routines') ;
addpath('./routines/solmat') ;
addpath('./../data/aggregate') ;
addpath('./../data/state') ;

SET.experiment = 'baseline' ;

%% Setup aggregate model and load aggregate data

a_setup_model

SET.a.param_names = cellstr(dyn_in.M_.param_names) ;

[num,txt,raw] = xlsread('USdata_2015.xlsx',3) ; 

date_vec = 1952:0.25:2015.25;

trim = find(date_vec==1984):find(date_vec==2015) ;

date_vec = date_vec(trim) ;

output          = num(trim,13) ;
debt            = num(trim,14)*4 ;
interest_rate   = num(trim,18) ;
inflation       = num(trim,20) ; 
house_prices    = num(trim,21) ;
employment      = num(trim,17) ;

y_data  = log(output) ;                
p_data  = log(house_prices);           
n_data  = log(employment) ;

[p_data1, p_data2]  = hptrend(p_data, .5);

dy_data = debt ./ output ;

[~, y_data]  = hptrend(y_data, 10^7);
[~, p_data]  = hptrend(p_data1, 10^7);
[temp, dy_data] = hptrend(dy_data, 10^7);
[~, n_data]  = hptrend(n_data, 10^7);

i_data   = (interest_rate - 2)/4/100 ; 
pai_data = (inflation - 2.5)/4/100;  

zlb_start  = find(date_vec==2009) ;

SET.zlb_start = zlb_start ;

load data_actual.mat

data_raw = data ;

a_n_data ; 
a_w_data ; 
a_c_data ; 

a_data = [ ...
    data(1,1:length(trim)) ; 
    data(2,1:length(trim)) - 0.005 ;
    nhat_data(1:length(trim))';
    dy_data' ;
    [0 [p_data(2:end)-p_data(1:end-1)]']
    what_data(1:length(trim))' ;
    chat_data(1:length(trim))' ] ;

SET.a.ss   = length(a_data(1,:)) ;
nobs       = length(a_data(:,1)) ;
SET.a.nobs = nobs ;

SET.a.EST.hidx        = 1:SET.a.nobs ;
SET.a.EST.hidx_no_obs = 2:SET.a.nobs ;

SET.a.EST.obs_ = [
    SET.variable.iobs
    SET.variable.paihat
    SET.variable.nhat
    SET.variable.dyobs
    SET.variable.dpobs
    SET.variable.dwobs
    SET.variable.dcobs
    ] ; 

a_gen_set_params(SET,dyn_in)

SET.a.data = a_data ;
SET.a.M_ = M_ ;
SET.a.oo_ = oo_ ;
SET.a.options_ = options_ ;
SET.a.variable = SET.variable ;

%% Shock decomposition

SET.a.params_to_estimate = ...
    {'alph', 'rho', 'rhow', ...
     'rhoz', 'rhom', 'rhoh', 'rhon', 'rhob', 'rhop', ...
     'sigmaz', 'sigmam', 'sigmah', 'sigman', 'sigmab', 'sigmap', 'sigmar','rhomMA','rhohMA'} ;

for qq=1:length(SET.a.params_to_estimate)
    tmp = find(contains(SET.a.param_names,SET.a.params_to_estimate{qq}),1) ;
    SET.a.params_to_estimate_idx(qq) = tmp ;
end

xx = SET.a.M_.params(SET.a.params_to_estimate_idx) ;

SET.a.tr_row = SET.tr_row ;
SET.a.zlb_val = SET.zlb_val ;
SET.a.pos_of_i = SET.pos_of_i ;

SET.ss = length(data(1,:)) ;

T_f   = zeros(1,length(data(1,:))) ;

zlb_t = zeros(1,SET.ss) ;
zlb_t(101:end) = 1 ;

T_f(zlb_t>0) =  round([6.2097    5.8613    5.0713    4.5748    4.3537    5.5664    7.0992 ...
    8.3829    5.6791    5.3383    8.8957   10.5846   10.9704   11.7437 ...
    12.0144   12.0463   10.7932    8.6387    7.1684    7.2150    6.1610 ...
    5.8534    5.1121    6.0688    6.5240  ]);

[~, Ct, Pt, Dt, Qt, Gt] = a_ll(xx, T_f, SET, a_data) ;

V  = zeros(nobs,nobs) ; 
H  = zeros(nobs,n_-1) ; 

SET.hidx = SET.a.EST.hidx ;

for ii=1:nobs ; H(ii,SET.a.EST.obs_(ii)) = 1 ; end

T_f_kf = ones(1,SET.ss) ; T_f_kf(T_f>0) = 0 ;
    
[x_T, n_T] = ks_(SET.a, Ct, Pt, Dt, H, V, a_data, T_f_kf) ;

x_T_1 = x_T ;
n_T_1 = n_T ;

%% Paths without ZLB

SC.t_f = zeros(1,length(data)) ;

[~, ~, ~, ~, Qt, Gt] = a_ll(xx, 0*T_f, SET, a_data) ;

y_ss_init = (eye(SET.variable.n_-1) - ...
    Qt(1:SET.variable.n_-1,1:SET.variable.n_-1,1)) \ Qt(1:SET.variable.n_-1,end,1)  ;

for ii=1:l_
    n_T       = n_T_1 ;
    vc        = 1:l_ ;
    vc(ii)    = [] ; 
    n_T(vc,:) = 0 ; % Select only iith shock

    y_sd  = [y_ss_init ; 1] ;
    
    for t=2:SET.ss
        y_sd(:,t) = Qt(:,:,t)*y_sd(:,t-1) + Gt(:,:,t)*n_T(:,t) ;
    end

    y_sh_dec{ii} = y_sd ;
end

obs_names{1} = 'iobs' ;
obs_names{2} = 'paihat' ;
obs_names{3} = 'nhat' ;
obs_names{4} = 'dyobs' ;
obs_names{5} = 'phat' ;
obs_names{6} = 'dwobs' ;
obs_names{7} = 'dcobs' ;

for jj=1:SET.a.nobs
    for ii=1:l_
        eval([obs_names{jj},'_mat(',int2str(ii),',:) = ',...
            'y_sh_dec{',int2str(ii),'}(SET.variable.',...
            obs_names{jj},',:) ;']);        
    end 
    eval([obs_names{jj},'_mat(',int2str(l_+1),',:) = ',...
            'x_T(SET.variable.',obs_names{jj},',:) - ',...
            'sum(',obs_names{jj},'_mat(1:',int2str(l_),',:)) ;']); 
end

%% Removing credit shocks

cf_start  = 2 ;

T_f = zeros(1,length(data)) ;
T_f(zlb_t>0) =  1*round([6.2097    5.8613    5.0713    4.5748    4.3537    5.5664    7.0992 ...
    8.3829    5.6791    5.3383    8.8957   10.5846   10.9704   11.7437 ...
    12.0144   12.0463   10.7932    8.6387    7.1684    7.2150    6.1610 ...
    5.8534    5.1121    6.0688    6.5240  ]);
SC.t_f = T_f ;

[~, ~, ~, ~, Qt, Gt] = a_ll(xx, T_f, SET, data) ;

y_ss_init = (eye(SET.variable.n_-1) - ...
    Qt(1:SET.variable.n_-1,1:SET.variable.n_-1,1)) \ Qt(1:SET.variable.n_-1,end,1)  ;

n_T       = n_T_1 ;
n_T2      = n_T_1 ;
n_T([3 4 5 6 7],cf_start:end) = 0 ; % credit shocks are em and eh in positions 1 and 2
n_T2([1 2],cf_start:end) = 0 ;  

y_sd  = [x_T(:,1) ; 1] ;
y_nozlb  = [x_T(:,1) ; 1] ;

for t=2:SET.ss
    y_sd(:,t) = Qt(:,:,1)*y_sd(:,t-1) + Gt(:,:,1)*n_T(:,t) ;
    y_nozlb(:,t) = Qt(:,:,1)*y_nozlb(:,t-1) + Gt(:,:,1)*n_T_1(:,t) ;
end

y_endog_dur = [x_T(:,1) ; 1] ;
y_chk       = [x_T(:,1) ; 1] ;

SET.maxchk = 100 ;

for t=2:SET.ss
    y_chk(:,t) = Qt(:,:,t)*y_chk(:,t-1) + Gt(:,:,t)*n_T_1(:,t) ;
    [y_endog_dur(:,t), zlb_f(t), z_f_T(t)] = ...
        zlb(SET, y_endog_dur(:,t-1), n_T_1(:,t)) ;
end

%%

figure; 

set(gcf, 'DefaultAxesLineWidth', 1.5);
set(gcf, 'DefaultLineLineWidth', 2);
set(gcf, 'DefaultAxesTickLabelInterpreter','latex'); 
set(gcf, 'DefaultLegendInterpreter','latex');
set(gcf, 'DefaultAxesFontSize',12);

subplot(2,2,1) ;

    plot(date_vec,1*x_T(SET.variable.dyobs,:),'LineWidth',2) ; hold on ;
    plot(date_vec(cf_start-1:1:end),1*y_sd(SET.variable.dyobs,cf_start-1:1:end),':','LineWidth',2) ;
    line([1980 2020],[0 0],'Color','k','LineWidth',1) ;
    xlim([1995 2015]) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('A. Debt to Income','Interpreter','latex')
    h2=legend('Data','Credit shocks only','location','southwest');
    set(h2,'Interpreter','latex');  

subplot(2,2,2) ;

    plot(date_vec,400*x_T(SET.variable.iobs,:),'Color',[     0    0.4470    0.7410],'LineWidth',2) ; hold on ;
    plot(date_vec,400*x_T(SET.variable.rf,:),'--','Color',[0.9290    0.6940    0.1250],'LineWidth',2) ; hold on ; 
    plot(date_vec(cf_start-1:1:end),400*y_sd(SET.variable.rf,cf_start-1:1:end),':','Color',[0.8500    0.3250    0.0980],'LineWidth',2) ;
	line([1980 2020],[0 0],'Color','k','LineWidth',1) ;
    xlim([1995 2015]) ;
    h=legend('Fed Funds', 'Natural','Natural, credit shocks only','location','northwest');
    set(h,'Interpreter','latex') ;
    ylim([-2 8]);
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('B. Interest Rates','Interpreter','latex')
    set(h,'FontSize',11)

subplot(2,2,3) ;

    plot(date_vec,100*x_T(SET.variable.nhat,:)-...
        100*x_T(SET.variable.nhat,find(date_vec==2008)),'LineWidth',2) ; hold on ;
    plot(date_vec(cf_start-1:1:end),100*y_sd(SET.variable.nhat,cf_start-1:1:end) - ...
        100*y_sd(SET.variable.nhat,find(date_vec==2008)),':','LineWidth',2) ;
    line([1980 2020],[0 0],'Color','k','LineWidth',1) ;
    xlim([1995 2015]) ;
	ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('C. Employment','Interpreter','latex')
  
subplot(2,2,4) ;

    plot(date_vec,400*x_T(SET.variable.paihat,:)+2,'LineWidth',2) ; hold on ;
    plot(date_vec(cf_start-1:1:end),400*y_sd(SET.variable.paihat,cf_start-1:1:end)+2,':','LineWidth',2) ;
	line([1980 2020],[0 0],'Color','k','LineWidth',1) ;
    xlim([1995 2015]) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('D. Inflation','Interpreter','latex')    
    
set(h,'FontSize',6) ;
set(h2,'FontSize',8) ;

%% No forward guidance

cf_start  = find(date_vec==2007.25) ; % at credit bust

n_T       = n_T_1 ;
n_T2      = n_T_1 ;
n_T2([1 2],cf_start:end) = 0 ; 

y_nozlb  = [x_T(:,1) ; 1] ;
y_nozlb_nocs  = [x_T(:,1) ; 1] ;

for t=2:SET.ss
    y_nozlb(:,t) = Qt(:,:,1)*y_nozlb(:,t-1) + Gt(:,:,1)*n_T_1(:,t) ;
    y_nozlb_nocs(:,t) = Qt(:,:,1)*y_nozlb_nocs(:,t-1) + Gt(:,:,1)*n_T2(:,t) ;
end

y_nocs  = [x_T(:,1) ; 1] ;
y_endog_dur = [x_T(:,1) ; 1] ;
y_endog_dur_nocs = [x_T(:,1) ; 1] ;

SET.maxchk=0 ;

for t=2:SET.ss
    y_nocs(:,t) = Qt(:,:,t)*y_nocs(:,t-1) + Gt(:,:,t)*n_T2(:,t) ;
    SET.maxchk = T_f(t) ; 
    [y_endog_dur(:,t), zlb_f(t), z_f_T(t)] = ...
        zlb(SET, y_endog_dur(:,t-1), n_T_1(:,t)) ;    
    [y_endog_dur_nocs(:,t), zlb_f(t), z_f_T_nocs(t)] = ...
        zlb(SET, y_endog_dur_nocs(:,t-1), n_T2(:,t)) ;    
end

figure; 
set(gcf,'PaperPosition',[0 0 11 3.5]) ;
set(gcf,'DefaultLineLineWidth', 3) ;
subplot(1,2,1) ; hold on ;
    plot(date_vec,100*x_T(SET.variable.nhat,:)-...
        100*x_T(SET.variable.nhat,find(date_vec==2008)),'LineWidth',3) ; hold on ;
    plot(date_vec,100*(y_endog_dur(SET.variable.nhat,1:1:end))-...
        100*y_endog_dur(SET.variable.nhat,find(date_vec==2008)),':','LineWidth',3) ;
    plot(date_vec,100*(y_endog_dur_nocs(SET.variable.nhat,1:1:end))-...
        100*y_endog_dur_nocs(SET.variable.nhat,find(date_vec==2008)),'-.','LineWidth',3) ;
    plot(date_vec,100*x_T(SET.variable.nhat,:)-...
        100*x_T(SET.variable.nhat,find(date_vec==2008)),'Color',[0, 0.4470,0.7410],'LineWidth',3) ; 
    line([1980 2020],[0 0],'Color','k','LineWidth',1) ;
    xlim([1995 2015]) ;
    
	ax = gca ;
    ax.YGrid='on';
    
    box('off') ;
    h=legend('Data','No FG, all shocks','No FG, no credit shocks','location','southwest') ;
    set(h,'Interpreter','latex');  
    title('A. Employment Response, No FG','Interpreter','latex')
    
    yticks(-18:6:6);
    ylim([-18 6]) ;    
	set(get(gcf,'CurrentAxes'),'FontName','Times New Roman','FontSize',13,'LineWidth',1.5)
    set(gca,'DefaultTextInterpreter','latex','TickLabelInterpreter','latex')       
subplot(1,2,2) ;

set(gcf,'DefaultLineLineWidth', 3) ;
    plot(date_vec,100*x_T(SET.variable.nhat,:)-...
        100*x_T(SET.variable.nhat,find(date_vec==2008)),'LineWidth',3) ; hold on ;
    plot(date_vec,100*(y_endog_dur(SET.variable.nhat,1:1:end)-...
        y_endog_dur_nocs(SET.variable.nhat,1:1:end)),':','LineWidth',3) ;
    plot(date_vec,100*x_T(SET.variable.nhat,:)-...
        100*x_T(SET.variable.nhat,find(date_vec==2008)),'Color',[0    0.4470    0.7410],'LineWidth',3) ; hold on ;
    
    line([1980 2020],[0 0],'Color','k','LineWidth',1) ;
    xlim([1995 2015]) ;
    
	ax = gca ;
    ax.YGrid='on';
    
    box('off') ;
    h2=legend('Data','Contribution of credit shocks only','location','southwest') ;
    set(h2,'Interpreter','latex') ;  
    title('B. Contribution of Credit Shocks, No FG','Interpreter','latex')

    yticks(-18:6:6);
    ylim([-18 6]) ;
    
	set(get(gcf,'CurrentAxes'),'FontName','Times New Roman','FontSize',13,'LineWidth',1.5)
    
    set(gca,'DefaultTextInterpreter','latex','TickLabelInterpreter','latex')       

set(h,'FontSize',10) ;
set(h2,'FontSize',10) ;

%% Table on aggregate effects of credit shocks

tableout=[] ;

tmpdat=1*x_T(SET.variable.debt,:)/exp(dyss)-...
    1*x_T(SET.variable.debt,find(date_vec==2007))/exp(dyss);
tmp2=1*y_sd(SET.variable.debt,:)/exp(dyss)-...
    1*y_sd(SET.variable.debt,find(date_vec==2007))/exp(dyss);
tmp = 1*(y_endog_dur(SET.variable.debt,:)/exp(dyss) - ...
    y_endog_dur_nocs(SET.variable.debt,:)/exp(dyss));
tmp = tmp-tmp(:,find(date_vec==2007)) ;

tableout=[tableout ; 
    [tmpdat(:,find(date_vec==2010))
    tmp2(:,find(date_vec==2010))
    tmp(:,find(date_vec==2010))]' ;
    [tmpdat(:,find(date_vec==2012.75))
    tmp2(:,find(date_vec==2012.75))
    tmp(:,find(date_vec==2012.75))]'] ;

tmpdat = 100*x_T(SET.variable.nhat,:)-...
    100*x_T(SET.variable.nhat,find(date_vec==2007));
tmp2 = 100*y_sd(SET.variable.nhat,:)-...
    100*y_sd(SET.variable.nhat,find(date_vec==2007));
tmp = 100*(y_endog_dur(SET.variable.nhat,:) - ...
    y_endog_dur_nocs(SET.variable.nhat,:));
tmp = tmp-tmp(:,find(date_vec==2007)) ;

tableout=[tableout ; 
    [tmpdat(:,find(date_vec==2010))
    tmp2(:,find(date_vec==2010))
    tmp(:,find(date_vec==2010))]' ;
    [tmpdat(:,find(date_vec==2012.75))
    tmp2(:,find(date_vec==2012.75))
    tmp(:,find(date_vec==2012.75))]'] ;

disp(' ') ;
disp('Table on aggregate effects of credit shocks:')
latex(tableout([1 3 2 4],:),'%.2f','nomath')

