%
% Aggregate IRF
%

clear all
clc

warning('Off','all') ;

path(pathdef)

addpath('./output') ;
addpath('./aggregate') ;
addpath('./regional') ;
addpath('./routines') ;
addpath('./routines/solmat') ;
addpath('./../data/aggregate') ;
addpath('./../data/state') ;

SET.experiment = 'baseline' ;

%% Setup model

SET.flexw = 0 ;

a_setup_model

x0 = SET.M_.params ;

SET.nsim = 200 ;

%% IRF 

SET.M_.params = x0 ;

dyn_in.M_       = SET.M_ ;
dyn_in.oo_      = SET.oo_ ;
dyn_in.options_ = SET.options_ ;
dyn_in.solve    = 1 ;
dyn_in.linearize_around_diff_y = 0 ;

mats = a_resolve_(SET,dyn_in) ;

Q = mats.mats.Q ; SET.mats.Q = Q;
G = mats.mats.G ; SET.mats.G = G;

yss = [(eye(n_-1) - Q(1:n_-1,1:n_-1)) \ Q(1:n_-1,end) ; 1] ;

etmat = zeros(SET.variable.l_,SET.nsim) ;

etmat(SET.variable.shock.em,2:6) = -22:5:-1 ; % Large credit shock

ysim(:,1,1) = yss ;
ysim_zlb(:,1,1) = yss ;

for tt=2:SET.nsim
    ysim(:,tt,1) = Q * ysim(:,tt-1,1) + G * etmat(:,tt) ;
    
     [ysim_zlb(:,tt,1), ~, z_f_T(tt)] = ...
        zlb(SET, ysim_zlb(:,tt-1,1), etmat(:,tt)) ;
end

%% IRF Flex Prices/Wages, High Uncertainty

SET.M_.params = x0 ;

tmp = find(contains(param_names,'rho'),1) ;
SET.M_.params(tmp) = 0.01 ;

tmp = find(contains(param_names,'rhow'),1) ;
SET.M_.params(tmp) = 0.01 ;

dyn_in.M_       = SET.M_ ;
dyn_in.oo_      = SET.oo_ ;
dyn_in.options_ = SET.options_ ;
dyn_in.solve    = 1 ;
dyn_in.linearize_around_diff_y = 0 ;

mats = a_resolve_(SET,dyn_in) ;

Q = mats.mats.Q ;
G = mats.mats.G ;
SET.mats.Q=Q;
SET.mats.G=G;

yss = [(eye(n_-1) - Q(1:n_-1,1:n_-1)) \ Q(1:n_-1,end) ; 1] ;

ysim(:,1,3) = yss ;
ysim_zlb(:,1,3) = yss ;

for tt=2:SET.nsim
    ysim(:,tt,3) = Q * ysim(:,tt-1,3) + G * etmat(:,tt) ;
    
     [ysim_zlb(:,tt,3), ~, z_f_T(tt)] = ...
        zlb(SET, ysim_zlb(:,tt-1,3), etmat(:,tt)) ;
end

%% IRF, Low Uncertainty

SET.M_.params = x0 ;

tmp = find(contains(param_names,'alph'),1) ;
SET.M_.params(tmp) = 5 ;

dyn_in.M_       = SET.M_ ;
dyn_in.oo_      = SET.oo_ ;
dyn_in.options_ = SET.options_ ;
dyn_in.solve    = 1 ;
dyn_in.linearize_around_diff_y = 0 ;

mats = a_resolve_(SET,dyn_in) ;

Q = mats.mats.Q ;
G = mats.mats.G ;
SET.mats.Q=Q;
SET.mats.G=G;

yss = [(eye(n_-1) - Q(1:n_-1,1:n_-1)) \ Q(1:n_-1,end) ; 1] ;

ysim(:,1,4) = yss ;
ysim_zlb(:,1,4) = yss ;

for tt=2:SET.nsim
    ysim(:,tt,4) = Q * ysim(:,tt-1,4) + G * etmat(:,tt) ;
    
     [ysim_zlb(:,tt,4), ~, z_f_T(tt)] = ...
        zlb(SET, ysim_zlb(:,tt-1,4), etmat(:,tt)) ;
end

%%

figure ;

set(gcf, 'DefaultAxesLineWidth', 1.5);
set(gcf, 'DefaultLineLineWidth', 3);
set(gcf, 'DefaultAxesTickLabelInterpreter','latex'); 
set(gcf, 'DefaultLegendInterpreter','latex');
set(gcf, 'DefaultAxesFontSize',24);
set(gcf,'PaperPosition',[0 0 17 10]) ;

subplot(2,3,1) ;
    plot(ysim(SET.variable.iobs,:,3)*400) ; hold on ;

    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    xlim([0 30]) ;
    ylim([-2 2]) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('A. Flexible Prices','Interpreter','latex'); 
    ylabel('Nominal Interest Rate','Interpreter','latex') ;

    set(gca,'DefaultTextInterpreter','latex','TickLabelInterpreter','latex') 
    
subplot(2,3,2) ;
    plot(ysim(SET.variable.iobs,:,1)*400) ; hold on ;
    plot(ysim_zlb(SET.variable.iobs,:,1)*400,'-.') ; hold on ;

    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    xlim([0 30]) ;
    ylim([-2 2]) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('B. Baseline','Interpreter','latex'); 
    h1 = legend('No ZLB', 'With ZLB');
    
    set(gca,'DefaultTextInterpreter','latex','TickLabelInterpreter','latex') 

subplot(2,3,3) ;
    plot(-201:-2,ysim(SET.variable.iobs,:,4)*400) ; hold on ;
    plot(ysim_zlb(SET.variable.iobs,:,4)*400,'-.') ; hold on ;

    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    xlim([0 30]) ;
    ylim([-2 2]) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('C. Low Uncertainty','Interpreter','latex'); 
    
    set(gca,'DefaultTextInterpreter','latex','TickLabelInterpreter','latex')     
    
subplot(2,3,4) ;
    plot(ysim(SET.variable.nhat,:,3)*100) ; hold on ;

    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    xlim([0 30]) ;
    ylim([-7 1]) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('D. Flexible Prices','Interpreter','latex'); 
    ylabel('Employment','Interpreter','latex') ;

    set(gca,'DefaultTextInterpreter','latex','TickLabelInterpreter','latex') 
    
subplot(2,3,5) ;
    plot(ysim(SET.variable.nhat,:,1)*100) ; hold on ;
    plot(ysim_zlb(SET.variable.nhat,:,1)*100,'-.') ; hold on ;

    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    xlim([0 30]) ;
    ylim([-7 1]) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('E. Baseline','Interpreter','latex'); 
    h2 = legend('No ZLB', 'With ZLB');
    
    set(gca,'DefaultTextInterpreter','latex','TickLabelInterpreter','latex') 

subplot(2,3,6) ;
    plot(-201:-2,ysim(SET.variable.nhat,:,4)*100) ; hold on ;
    plot(ysim_zlb(SET.variable.nhat,:,4)*100,'-.') ; hold on ;

    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    xlim([0 30]) ;
    ylim([-7 1]) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('F. Low Uncertainty','Interpreter','latex'); 
    
    set(gca,'DefaultTextInterpreter','latex','TickLabelInterpreter','latex')     
    
 set(findall(gcf,'-property','FontSize'),'FontSize',24) ;
    set(h1,'Interpreter','latex','FontSize',18);  
    set(h2,'Interpreter','latex','FontSize',18);  
 