%
% State-Level IRF
%

clear all
clc

warning('Off','all') ;

path(pathdef)

addpath('./output') ;
addpath('./aggregate') ;
addpath('./regional') ;
addpath('./routines') ;
addpath('./routines/solmat') ;
addpath('./../data/aggregate') ;
addpath('./../data/state') ;

SET.experiment = 'baseline' ;

SET.statetrend = 1 ;
SET.popweights = 1 ;
SET.debtinreg  = 0 ;
SET.removetop5states = 0 ;

%% Set up matrices

r_setup_model

SET.r.param_names = cellstr(dyn_in.M_.param_names) ;

r_gen_set_params(SET,dyn_in)

NQuarters     = 12*4 ;
Q_idx         = 1:4:NQuarters ;
SET.r.TT(Q_idx) = 1 ;
SET.r.Nstate    = 51 ;

r_data = [];

r_load_data

SET.r.nobs = length(r_data{1}(:,1));
SET.r.EST.obs_ = [ ...
    SET.variable.spendobs_
    SET.variable.nobs_
    SET.variable.wobs_
    SET.variable.debtobs_
    SET.variable.pobs_] ;

SET.r.data = r_data ;
SET.r.M_ = M_ ;
SET.r.oo_ = oo_ ;
SET.r.options_ = options_ ;
SET.r.variable = SET.variable ;

x0 = SET.r.M_.params ;

%% IRF at baseline posterior median

SET.nsim = 200 ;

SET.r.M_.params = x0 ;

dyn_in.M_       = SET.r.M_ ;
dyn_in.oo_      = SET.r.oo_ ;
dyn_in.options_ = SET.r.options_ ;
dyn_in.solve    = 1 ;
dyn_in.linearize_around_diff_y = 0 ;

mats = r_resolve_(SET,dyn_in) ;

Q = mats.mats.Q ;
G = mats.mats.G ;

n_  = SET.variable.n_ ;
yss = [(eye(n_-1) - Q(1:n_-1,1:n_-1)) \ Q(1:n_-1,end) ; 1] ;

etmat = zeros(SET.variable.l_,SET.nsim) ;

etmat(SET.variable.shock.em,2) = -3 ;

ysim      = zeros(n_,SET.nsim) ;
ysim(:,1,1) = yss ;

for tt=2:SET.nsim
    ysim(:,tt,1) = Q * ysim(:,tt-1,1) + G * etmat(:,tt) ;
end

%% High alpha

SET.r.M_.params = x0 ;

tmp = find(contains(param_names,'alph'),1) ;
SET.r.M_.params(tmp) = 5 ;

dyn_in.M_       = SET.r.M_ ;
dyn_in.oo_      = SET.r.oo_ ;
dyn_in.options_ = SET.r.options_ ;
dyn_in.solve    = 1 ;
dyn_in.linearize_around_diff_y = 0 ;

mats = r_resolve_(SET,dyn_in) ;

Q = mats.mats.Q ;
G = mats.mats.G ;

yss = [(eye(n_-1) - Q(1:n_-1,1:n_-1)) \ Q(1:n_-1,end) ; 1] ;

ysim(:,1,2) = yss ;

for tt=2:SET.nsim
    ysim(:,tt,2) = Q * ysim(:,tt-1,2) + G * etmat(:,tt) ;
end

%% Flexible prices

SET.r.M_.params = x0 ;

tmp = find(contains(param_names,'rho'),1) ;
SET.r.M_.params(tmp) = eps ;

tmp = find(contains(param_names,'rhow'),1) ;
SET.r.M_.params(tmp) = eps ;

dyn_in.M_       = SET.r.M_ ;
dyn_in.oo_      = SET.r.oo_ ;
dyn_in.options_ = SET.r.options_ ;
dyn_in.solve    = 1 ;
dyn_in.linearize_around_diff_y = 0 ;

mats = r_resolve_(SET,dyn_in) ;

Q = mats.mats.Q ;
G = mats.mats.G ;

yss = [(eye(n_-1) - Q(1:n_-1,1:n_-1)) \ Q(1:n_-1,end) ; 1] ;

ysim(:,1,3) = yss ;

for tt=2:SET.nsim
    ysim(:,tt,3) = Q * ysim(:,tt-1,3) + G * etmat(:,tt) ;
end

%%

figure

set(gcf, 'DefaultAxesLineWidth', 1.5);
set(gcf, 'DefaultLineLineWidth', 2);
set(gcf, 'DefaultAxesTickLabelInterpreter','latex'); 
set(gcf, 'DefaultLegendInterpreter','latex');
set(gcf,'PaperPosition',[0 0 13 14]) ;

subplot(4,3,1) ; hold on ;
    plot(1/4*(ysim(SET.variable.debt2,:,3)));
    plot(1/4*(ysim(SET.variable.asset,:,3)),'-.');
    xlim([0 30]) ;
    ylim([2.6 3]/4) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('A. Flexible Prices','Interpreter','latex','FontSize',17); 
    ylabel('Debt, Liquid Assets','Interpreter','latex','FontSize',17) ;
    h = legend('Debt / $Y_{ss}$', 'Liquid Assets / $Y_{ss}$','location','northeast');
    set(h,'Interpreter','latex');  
    
subplot(4,3,7) ; hold on ;
    plot(100*ysim(SET.variable.what,:,3))
    xlim([0 30]) ;
    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    ylim([-0.4 0.0500001]) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('G. Flexible Prices','Interpreter','latex','FontSize',17); 
    ylabel('Wages','Interpreter','latex','FontSize',17) ;
    set(h,'Interpreter','latex');  
    
subplot(4,3,10) ; hold on ;
    plot(100*ysim(SET.variable.nhat,:,3))
    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    xlim([0 30]) ;
    ylim([-1.0 0.2000001]) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('J. Flexible Prices','Interpreter','latex','FontSize',17); 
    ylabel('Employment','Interpreter','latex','FontSize',17) ;
    set(h,'Interpreter','latex');  
    
subplot(4,3,4) ; hold on ;
    plot(100*(ysim(SET.variable.c,:,3)-ysim(SET.variable.c,1,3)))
    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    xlim([0 30]) ;
    ylim([-1.0 0.2000001]) ;
    box('off') ;
    ax = gca ;
    ax.YGrid='on';
    title('D. Flexible Prices','Interpreter','latex','FontSize',17); 
    ylabel('Consumption','Interpreter','latex','FontSize',17) ;
    set(h,'Interpreter','latex');  

subplot(4,3,3) ; hold on ;
    plot(1/4*(ysim(SET.variable.debt2,:,2)));
    plot(1/4*(ysim(SET.variable.asset,:,2)),'-.');
    xlim([0 30]) ;
    ylim([2.6 3]/4) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('C. Low Uncertainty','Interpreter','latex','FontSize',17); 

subplot(4,3,9) ; hold on ;
    plot(100*ysim(SET.variable.what,:,2))
    xlim([0 30]) ;
    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    ylim([-0.4 0.0500001]) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('I. Low Uncertainty','Interpreter','latex','FontSize',17); 
    set(h,'Interpreter','latex');  
    
subplot(4,3,12) ; hold on ;
    plot(100*ysim(SET.variable.nhat,:,2))
    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    xlim([0 30]) ;
    ylim([-1.0 0.2000001]) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('L. Low Uncertainty','Interpreter','latex','FontSize',17); 
    set(h,'Interpreter','latex');  
    
subplot(4,3,6) ; hold on ;
    plot(100*(ysim(SET.variable.c,:,2)-ysim(SET.variable.c,1,2)))
    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    xlim([0 30]) ;
    ylim([-1.0 0.2000001]) ;
    box('off') ;
    ax = gca ;
    ax.YGrid='on';
    title('F. Low Uncertainty','Interpreter','latex','FontSize',17); 
    set(h,'Interpreter','latex');      

subplot(4,3,2) ; hold on ;
    plot(1/4*(ysim(SET.variable.debt2,:,1)));
    plot(1/4*(ysim(SET.variable.asset,:,1)),'-.');
    xlim([0 30]) ;
    ylim([2.6 3]/4) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('B. Baseline','Interpreter','latex','FontSize',17); 
    
subplot(4,3,8) ; hold on ;
    plot(100*ysim(SET.variable.what,:,1))
    xlim([0 30]) ;
    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    ylim([-0.4 0.0500001]) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('H. Baseline','Interpreter','latex','FontSize',17); 
    set(h,'Interpreter','latex');  
    
subplot(4,3,11) ; hold on ;
    plot(100*ysim(SET.variable.nhat,:,1))
    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    xlim([0 30]) ;
    ylim([-1.0 0.2000001]) ;
    ax = gca ;
    ax.YGrid='on';
    box('off') ;
    title('K. Baseline','Interpreter','latex','FontSize',17); 
    set(h,'Interpreter','latex');  
    
subplot(4,3,5) ; hold on ;
    plot(100*(ysim(SET.variable.c,:,1)-ysim(SET.variable.c,1,1)))
    line([0 30],[0 0],'LineWidth',1,'Color','k') ;
    xlim([0 30]) ;
    ylim([-1.0 0.2000001]) ;
    box('off') ;
    ax = gca ;
    ax.YGrid='on';
    title('E. Baseline','Interpreter','latex','FontSize',17); 
    set(h,'Interpreter','latex');          


