function [out, Ct, Pt, Dt, Qt, Gt, U] = r_ll(xx, mats, SET, data)
%function out = ll(params, SET, data)
%
% Calculates the log-likelihood for Gaussian sequence given a mean and
% variance sequence.
%
% State space representation:
% 
% S_t = C(t) + P(t) S_{t-1} + D(t) n_t : State-state equation
% X_t = H S_t + v_t : observation equation

SET  = SET.r ;

n_   = SET.variable.n_ ; 
nobs = SET.nobs ;

%% Re-solve model with xx parameters

SET.M_.params(SET.params_to_estimate_idx) = xx ;

if sum(SET.M_.params(SET.params_to_estimate_idx)<0)>0
    out=-1e12;
    return
end

if isempty(mats.mats.Q) ; out = -10e10 ; return ; end

Qt = repmat(mats.mats.Q,[1 1 SET.ss]) ;
Gt = repmat(mats.mats.G,[1 1 SET.ss]) ; 

%% Extract matrices for Kalman filter

Ct = Qt(1:n_-1,end,:) ;
Pt = Qt(1:n_-1,1:n_-1,:) ;
Dt = Gt(1:n_-1,:,:) ;

H = zeros(nobs,n_-1) ; 

for i=1:nobs
    H(i,SET.EST.obs_(i)) = 1 ;
end

TT = SET.TT ; 

[U, St, flag] = kf_(SET, Ct, Pt, Dt, H, data, TT) ;

if flag ; out=-1e12 ; return ; end

out = 0;

for t=1:SET.ss
    hidx = SET.EST.hidx ;
    if TT(t)==0 ; hidx = SET.EST.hidx_no_obs ; end 

    out = out - 1*(TT(t)>0)*(nobs/2) * log(2*pi) - ...
        (1/2) * log(det(St(hidx,hidx,t))) - ...
        (1/2) * U(hidx,t)' / St(hidx,hidx,t) * U(hidx,t) ;
end

if abs(imag(out))>0
    out=-1e12;
end
