function [x_T, n_T] = ks_(SET, Ct, Pt, Dt, H, V, data, TT)
%
% Kalman smoother
%

n    = SET.variable.n_ ;
ss   = SET.ss ;
nobs = SET.nobs ;

%% Initialize

Omega  = SET.Omega ;
DOmega = Dt(:,:,1)*Omega*Dt(:,:,1)' ;
[Sigma, ~]  = gmres((eye((n-1)^2)-(kron(Pt(:,:,1),Pt(:,:,1)))),...
    DOmega(:),[],[],[],[],[],zeros(length(DOmega(:)),1)) ;
Sigma       = reshape(Sigma,n-1,n-1) ;  

S     = zeros(nobs,nobs,ss) ;
y_til = zeros(nobs,ss) ; 
x     = zeros(n-1,nobs) ;
P     = zeros(n-1,n-1,nobs) ;
K     = zeros(n-1,nobs,ss) ;

Ph = Sigma ;
xh = ((eye(n-1) - Pt(:,:,1))) \ Ct(:,:,1)  ;

P(:,:,1) = Ph ;
x(:,1)   = xh ;

Pt(:,:,ss+1)=Pt(:,:,ss-1);

%% Obtain Kalman filter matrices with time T information

for t=1:ss
    hidx = SET.EST.hidx ;
    if TT(t)==0 ; hidx = SET.EST.hidx_no_obs ; end 

    % Predict
    x(:,t) = Ct(:,:,t)+Pt(:,:,t)*xh ;
    P(:,:,t) = Pt(:,:,t)*Ph*Pt(:,:,t)' + ...
        Dt(:,:,t)*Omega*Dt(:,:,t)' ;
    
    % Update
    y_til(hidx,t) = data(hidx,t) - H(hidx,:)*x(:,t) ;
    S(hidx,hidx,t) = H(hidx,:)*P(:,:,t)*H(hidx,:)' ;
    K(:,hidx,t) = P(:,:,t)*H(hidx,:)' / S(hidx,hidx,t) ;
    xh = x(:,t) + K(:,hidx,t)*y_til(hidx,t);
    Ph = (eye(n-1) - K(:,hidx,t)*H(hidx,:))*P(:,:,t) ; 
end

%% Kalman smoother with time T information

r_T(:,ss+1) = zeros(n-1,1) ;

for t=ss:-1:1
    hidx = SET.EST.hidx ;
    if TT(t)==0 ; hidx = SET.EST.hidx_no_obs ; end 

    r_T(:,t) = H(hidx,:)' / S(hidx,hidx,t) ...
        * (data(hidx,t) - H(hidx,:)*x(:,t) ) + ...
        (eye(n-1) - K(:,hidx,t)*H(hidx,:))' * Pt(:,:,t+1)' * r_T(:,t+1) ;

    x_T(:,t) = x(:,t) + P(:,:,t) * r_T(:,t) ;

    n_T(:,t) = Dt(:,:,t)' * r_T(:,t) ;
end


