function out = mcmc(SET, c, x0, runs)
% 
% Runs the Markov Chain Monte Carlo
%

%% Parameters

nparam = length(x0);

N     = SET.EST.N ; 
df    = SET.EST.df ; 

pop = 1:nparam ;

kk_xx  = 0 ; 
acc_xx = 0 ;

%% Posterior

post_for_minimizer = @(x) post(x,SET) ;
post_neg = @(x) -post_for_minimizer(x) ;

%% Settings for durations and parameters

Prop_xx      = zeros(N,length(x0)) ;
Prop_xx(1,:) = x0' ; 
xx_out       = zeros(N,length(x0)) ;

xx_out(1,:)   = Prop_xx(1,:)';

%% MCMC

denominator = post_neg(xx_out(1,:)) ;

for j=2:N
%% xx-block

xx_prop     = xx_out(j-1,:) ;
blocksize   = ceil(nparam*rand) ;
block       = randsample(pop,blocksize) ;

xx_prop(1,block) = xx_out(j-1,block) + ...
    (c(block,block)*trnd(df,blocksize,1))' ; % multivariate-t with df degrees of freedom

Prop_xx(j,:) = xx_prop(1,:) ;

if (min(xx_prop(:))<0)>=1
    alpha2 = 0;
    ratio2 = 0;
else
    numerator  = post_neg(xx_prop) ;
    ratio2     = exp(-numerator + denominator) ;
    alpha2     = min(ratio2,1) ;
end

if isnan(ratio2) ; alpha2 = 0; end

if rand <= alpha2
   xx_out(j,:) = xx_prop;
   kk_xx       = kk_xx + 1;
   denominator = numerator ;
else
    xx_out(j,:) = xx_out(j-1,:);
end

acc_xx= 100*kk_xx/j;

if mod(j/10,1)==0 
    fprintf(sprintf('Chain %1.0f .... %2.2f per cent done\n   ***** accept rates: x, %2.3f\n',runs,100*j/N,acc_xx)) ;
end

if mod(j/100,1)==0 
    prctile(xx_out(j-99:j,:),[50])
    denominator
end

if mod(j/(N/SET.EST.numbersave),1)==0
    eval(['save chain_', int2str(runs), ';']) ;
end

end
%% Outputs

out.xx_out  = xx_out ;
out.acc_rates = [acc_xx, N] ;

