%
% Estimation
%

clear all
clc

warning('Off','all') ;

path(pathdef)

addpath('./output') ;
addpath('./aggregate') ;
addpath('./regional') ;
addpath('./routines') ;
addpath('./routines/solmat') ;
addpath('./../data/aggregate') ;
addpath('./../data/state') ;

SET.experiment = 'baseline' ;

%%

SET.flexw           = 0 ;
SET.estimate_tr     = 0 ;
SET.aweight         = 0.5 ; % Weight on aggregate likelihood
SET.statetrend      = 1 ;
SET.popweights      = 1 ;
SET.debtinreg       = 0 ;
SET.estimate_MA     = 0 ;
SET.removetop5states = 0 ;

if strcmpi(SET.experiment,'baseline_gammaat0') ;    SET.estimate_MA = 1 ; end
if strcmpi(SET.experiment,'agg_only') ;             SET.aweight = 1.0 ; end
if strcmpi(SET.experiment,'reg_only') ;             SET.aweight = 0.0 ; end
if strcmpi(SET.experiment,'reg_only_agg') ;         SET.aweight = 1.0 ; end
if strcmpi(SET.experiment,'estimated_taylor_rule'); SET.estimate_tr = 1 ; end

%% Set up regional matrices and data

r_setup_model

SET.r.param_names = cellstr(dyn_in.M_.param_names) ;

r_gen_set_params(SET,dyn_in)

Nyears        = 17 ;
NQuarters     = Nyears*4 ;
Q_idx         = 1:4:NQuarters ;
SET.r.TT(Q_idx) = 1 ;
SET.r.Nstate    = 51 ;

r_data = [];

r_load_data

SET.r.nobs = length(r_data{1}(:,1));
SET.r.EST.obs_ = [ ...
    SET.variable.spendobs_
    SET.variable.nobs_
    SET.variable.wobs_
    SET.variable.debtobs_
    SET.variable.pobs_] ;

SET.r.data = r_data ;
SET.r.M_ = M_ ;
SET.r.oo_ = oo_ ;
SET.r.options_ = options_ ;
SET.r.variable = SET.variable ;

%% Set up aggregate matrices and data

a_setup_model

SET.a.param_names = cellstr(dyn_in.M_.param_names) ;

[num,txt,raw] = xlsread('USdata_2015.xlsx',3) ; 

date_vec = 1952:0.25:2015.25;

trim = find(date_vec==1984):find(date_vec==2015) ;

date_vec = date_vec(trim) ;

output          = num(trim,13) ;
debt            = num(trim,14)*4 ;
house_prices    = num(trim,21) ;

y_data  = log(output) ;                
p_data  = log(house_prices); 

[p_data1, p_data2]  = hptrend(p_data, .5);
dy_data         = debt ./ output ;
[~, y_data]     = hptrend(y_data, 10^7);
[~, p_data]     = hptrend(p_data1, 10^7);
[temp, dy_data] = hptrend(dy_data, 10^7);

zlb_start  = find(date_vec==2009) ;

SET.zlb_start = zlb_start ;

load data_actual.mat

data_raw = data ;

a_n_data ; 
a_w_data ; 
a_c_data ; 

a_data = [ ...
    data(1,1:length(trim)) ; 
    data(2,1:length(trim)) - 0.005 ;
    nhat_data(1:length(trim))';
    dy_data' ;
    [0 [p_data(2:end)-p_data(1:end-1)]']
    what_data(1:length(trim))' ;
    chat_data(1:length(trim))' ] ;

SET.a.ss   = length(a_data(1,:)) ;
nobs       = length(a_data(:,1)) ;
SET.a.nobs = nobs ;

SET.a.EST.hidx        = 1:SET.a.nobs ;
SET.a.EST.hidx_no_obs = 2:SET.a.nobs ;

SET.a.EST.obs_ = [
    SET.variable.iobs
    SET.variable.paihat
    SET.variable.nhat
    SET.variable.dyobs
    SET.variable.dpobs
    SET.variable.dwobs
    SET.variable.dcobs
    ] ; 

a_gen_set_params(SET,dyn_in)

SET.a.data = a_data ;
SET.a.M_ = M_ ;
SET.a.oo_ = oo_ ;
SET.a.options_ = options_ ;
SET.a.variable = SET.variable ;

figure;
subplot(3,3,1);
plot(a_data(1,:),'LineWidth',2)
title('Fed Funds Rate') ;
subplot(3,3,2);
plot(a_data(2,:),'LineWidth',2)
title('Inflation') ;
subplot(3,3,3);
plot(a_data(3,:),'LineWidth',2)
title('Employment') ;
subplot(3,3,4);
plot(a_data(4,:),'LineWidth',2)
title('Debt') ;
subplot(3,3,5);
plot(a_data(5,:),'LineWidth',2)
title('House Prices') ;
subplot(3,3,6);
plot(a_data(6,:),'LineWidth',2)
title('Wages') ;
subplot(3,3,7);
plot(a_data(7,:),'LineWidth',2)
title('Consumption') ;
subplot(3,3,8);
toplot = round([6.2097    5.8613    5.0713    4.5748    4.3537   ...
    5.5664    7.0992    8.3829    5.6791    5.3383    8.8957   ...
    10.5846   10.9704   11.7437   12.0144   12.0463   10.7932  ...
    8.6387    7.1684    7.2150    6.1610 ...
    5.8534    5.1121    6.0688    6.5240  ]);
bar(2009:0.25:2015,toplot);
ylim([0 14]) ;
title('ZLB Durations') ;

%% fmincon to get hessian

clearvars -except SET

SET.r.ss     = length(SET.r.data{1}(1,:)) ;

SET.r.params_to_estimate = ...
    {'alph', 'rho', 'rhow', ...
     'rhoz', 'rhom', 'rhoh', 'rhon', 'rhob', ...
     'sigmaz', 'sigmam', 'sigmah', 'sigman', 'sigmab'} ;
 
SET.a.params_to_estimate = ...
    {'alph', 'rho', 'rhow', ...
     'rhoz', 'rhom', 'rhoh', 'rhon', 'rhob', 'rhop', ... 
     'sigmaz', 'sigmam', 'sigmah', 'sigman', 'sigmab', 'sigmap', 'sigmar'} ;
 
SET.r.params_to_estimate_idx = [] ; 
SET.a.params_to_estimate_idx = [] ;
 
for qq=1:length(SET.r.params_to_estimate)
    tmp = find(contains(SET.r.param_names,SET.r.params_to_estimate{qq}),1) ;
    SET.r.params_to_estimate_idx(qq) = tmp ;
end

for qq=1:length(SET.a.params_to_estimate)
    tmp = find(contains(SET.a.param_names,SET.a.params_to_estimate{qq}),1) ;
    SET.a.params_to_estimate_idx(qq) = tmp ;
end

SET.r.EST.hidx = 1:SET.r.nobs ;
SET.r.EST.hidx_no_obs = [] ;

x0 = [ ...
    SET.r.M_.params(SET.r.params_to_estimate_idx)
    SET.a.M_.params(SET.a.params_to_estimate_idx(4:end))];

r_gen_post_fn(SET.r.Nstate,SET)

post_for_minimizer = @(x) post(x,SET) ;
post_for_minimizer = @(x) -post_for_minimizer(x) ;

post_for_minimizer(x0)

load hess_proposal
x0_mcmc = x0 ;

%% MCMC

delete(gcp('nocreate'))

SET.EST.chains = 2 ;
SET.EST.kappa  = 0.5 ;
SET.EST.N      = 100000 ;
SET.EST.df     = 16 ;
SET.init_Sigma  = 0 ;
SET.EST.numbersave = 10 ;

c = SET.EST.kappa * inv(hess) ; % Use fmincon Hessian
c = chol(c)' ;

maxproc = SET.EST.chains ;
parpool(maxproc)

clear params_x acc_rates
clc

parfor runs=1:maxproc
    warning('Off','all') ;
    
    out = mcmc(SET, c, x0_mcmc, runs) ;
    
    params_x(:,:,runs) = out.xx_out ;
    acc_rates(runs,:)  = out.acc_rates ;
end

delete(gcp('nocreate'))

clear date

eval(['save ./output/posterior_save_', date, '_', SET.experiment,'.mat;']);

%% Stack the (trimmed) chain outputs

maxproc = SET.EST.chains ;

SET.EST.burn_in = 0.5 ;

params_x_   = [] ;

for runs=1:maxproc
    params_x_ = [params_x_ ; ...
        params_x(round(length(params_x)*SET.EST.burn_in):end,:,runs)] ; %#ok<AGROW>
end

%% Chain diagnostics

sq_R_x = [];

pvec=1:26;

count = 1 ;

yvec = 500:500:length(params_x(:,1,1)) ;

for j=yvec
    sq_R_x(:,count) = ...
        mh_diag(params_x(1:j,pvec,:)) ;
    count = count+1 ;
end

figure ;
set(gcf,'PaperUnits','inches','PaperPosition',[0 0 5 3]) ;
    plot(yvec,sq_R_x','k','LineWidth',1) ;
    line([0 j],[1.1 1.1],'LineWidth',1,'color','r','LineStyle','--') ;
    line([0 j],[1 1],'LineWidth',1,'color','k') ;
    box('on') ; grid ;
    xlabel('Length of chain') ;
    ylim([1 3]) ;
    ylabel('R^2 diagnostic') ;
set(findall(gcf,'-property','FontSize'),'FontSize',9) ;
