
clear all
clc

warning('Off','all') ;

path(pathdef)

addpath('./output') ;
addpath('./aggregate') ;
addpath('./regional') ;
addpath('./routines') ;
addpath('./routines/solmat') ;
addpath('./../data/aggregate') ;
addpath('./../data/state') ;

SET.experiment = 'baseline' ;

%%

SET.flexw           = 0 ;
SET.estimate_tr     = 0 ;
SET.aweight         = 0.5 ; % Weight on aggregate likelihood
SET.statetrend      = 1 ;
SET.popweights      = 1 ;
SET.debtinreg       = 0 ;
SET.estimate_MA     = 0 ;
SET.removetop5states = 0 ;

%if strcmpi(SET.experiment,'statetrend') ; SET.statetrend = 1 ; end
if strcmpi(SET.experiment,'agg_only') ; SET.aweight = 1.0 ; end
if strcmpi(SET.experiment,'reg_only') ; SET.aweight = 0.0 ; end
if strcmpi(SET.experiment,'reg_only_agg') ; SET.aweight = 1.0 ; end
if strcmpi(SET.experiment,'estimated_taylor_rule') ; SET.estimate_tr = 1 ; end

%% Set up regional matrices and data

r_setup_model

SET.r.param_names = cellstr(dyn_in.M_.param_names) ;

r_gen_set_params(SET,dyn_in)

Nyears        = 17 ;
NQuarters     = Nyears*4 ;
Q_idx         = 1:4:NQuarters ;
SET.r.TT(Q_idx) = 1 ;
SET.r.Nstate    = 51 ;

r_data = [];

r_load_data

SET.r.nobs = length(r_data{1}(:,1));
SET.r.EST.obs_ = [ ...
    SET.variable.spendobs_
    SET.variable.nobs_
    SET.variable.wobs_
    SET.variable.debtobs_
    SET.variable.pobs_] ;

SET.r.data = r_data ;
SET.r.M_ = M_ ;
SET.r.oo_ = oo_ ;
SET.r.options_ = options_ ;
SET.r.variable = SET.variable ;

%%

SET.nsim = 5000 ;

n_  = SET.variable.n_ ;
yss = [(eye(n_-1) - Q(1:n_-1,1:n_-1)) \ Q(1:n_-1,end) ; 1] ;

%load shocks_for_smats

simmat=[];

for ii=1:51
    etmat = randn(SET.variable.l_,SET.nsim) ;
    
    ysim      = zeros(n_,SET.nsim) ;
    ysim(:,1) = yss ;
    
    for tt=2:SET.nsim
        ysim(:,tt) = Q * ysim(:,tt-1) + G * etmat(:,tt-1) ;
    end
    
    simmat{ii} = ysim ;
end

SET.EST.obs_ = [ ...
    SET.variable.spendobs_
    SET.variable.nobs_
    SET.variable.wobs_
    SET.variable.debtobs_
    SET.variable.pobs_] ;

for ii=1:51
    datasim{ii} = simmat{ii}(SET.EST.obs_,1:end) ;
end

for tt=1:Nyears
    for jj=1:Nstates
        c_tmp_sim(tt,jj) = (1/100)*datasim{jj}(1,Q_idx(tt)) ;
        n_tmp_sim(tt,jj) = (1/100)*datasim{jj}(2,Q_idx(tt)) ;
        w_tmp_sim(tt,jj) = (1/100)*datasim{jj}(3,Q_idx(tt)) ;
        d_tmp_sim(tt,jj) = datasim{jj}(4,Q_idx(tt)) ;
        q_tmp_sim(tt,jj) = (1/10)*datasim{jj}(5,Q_idx(tt)) ;
        
    end
    
    SET.TT(Q_idx(tt)) = 1 ;
end

c_vec = c_tmp_sim(:) ;
n_vec = n_tmp_sim(:) ;
w_vec = w_tmp_sim(:) ;
d_vec = d_tmp_sim(:) ;
q_vec = q_tmp_sim(:) ;

simmat = [c_vec n_vec w_vec d_vec q_vec] ;

%% Change alpha

tmp = find(contains(param_names,'alph'),1) ;
SET.r.M_.params(tmp) = 10 ;

dyn_in.M_       = SET.r.M_ ;
dyn_in.oo_      = SET.r.oo_ ;
dyn_in.options_ = SET.r.options_ ;
dyn_in.solve    = 1 ;
dyn_in.linearize_around_diff_y = 0 ;

mats = r_resolve_(SET,dyn_in) ;

Q = mats.mats.Q ;
G = mats.mats.G ;

simmat=[];

for ii=1:51
    etmat = randn(SET.variable.l_,SET.nsim) ;
    %etmat = shocks(:,(1+(ii-1)*5):(ii*5))' ;
    
    ysim      = zeros(n_,SET.nsim) ;
    ysim(:,1) = yss ;
    
    for tt=2:SET.nsim
        ysim(:,tt) = Q * ysim(:,tt-1) + G * etmat(:,tt-1) ;
    end
    
    simmat{ii} = ysim ;
end

SET.EST.obs_ = [ ...
    SET.variable.spendobs_
    SET.variable.nobs_
    SET.variable.wobs_
    SET.variable.debtobs_
    SET.variable.pobs_] ;

for ii=1:51
    datasim{ii} = simmat{ii}(SET.EST.obs_,1:end) ;
end

for tt=1:Nyears
    for jj=1:Nstates
        c_tmp_sim(tt,jj) = (1/100)*datasim{jj}(1,Q_idx(tt)) ;
        n_tmp_sim(tt,jj) = (1/100)*datasim{jj}(2,Q_idx(tt)) ;
        w_tmp_sim(tt,jj) = (1/100)*datasim{jj}(3,Q_idx(tt)) ;
        d_tmp_sim(tt,jj) = datasim{jj}(4,Q_idx(tt)) ;
        q_tmp_sim(tt,jj) = (1/10)*datasim{jj}(5,Q_idx(tt)) ;
        
    end
    
    SET.TT(Q_idx(tt)) = 1 ;
end

c_vec = c_tmp_sim(:) ;
n_vec = n_tmp_sim(:) ;
w_vec = w_tmp_sim(:) ;
d_vec = d_tmp_sim(:) ;
q_vec = q_tmp_sim(:) ;

simmat = [c_vec n_vec w_vec d_vec q_vec] ;