
clear all
clc

warning('Off','all') ;

path(pathdef)

addpath('./output') ;
addpath('./aggregate') ;
addpath('./regional') ;
addpath('./routines') ;
addpath('./routines/solmat') ;
addpath('./../data/aggregate') ;
addpath('./../data/state') ;

SET.experiment = 'baseline' ;

%%

SET.flexw           = 0 ;
SET.estimate_tr     = 0 ;
SET.aweight         = 0.5 ; % Weight on aggregate likelihood
SET.statetrend      = 1 ;
SET.popweights      = 1 ;
SET.debtinreg       = 0 ;
SET.estimate_MA     = 0 ;
SET.removetop5states = 0 ;

%% Set up regional matrices and data

r_setup_model

SET.r.param_names = cellstr(dyn_in.M_.param_names) ;

r_gen_set_params(SET,dyn_in)

NYears        = 17 ;
NQuarters     = NYears*4 ;
Q_idx         = 1:4:NQuarters ;
SET.r.TT(Q_idx) = 1 ;
SET.r.Nstate    = 51 ;

r_data = [];

r_load_data

SET.r.nobs = length(r_data{1}(:,1));
SET.r.EST.obs_ = [ ...
    SET.variable.spendobs_
    SET.variable.nobs_
    SET.variable.wobs_
    SET.variable.debtobs_
    SET.variable.pobs_] ;

SET.r.data = r_data ;
SET.r.M_ = M_ ;
SET.r.oo_ = oo_ ;
SET.r.options_ = options_ ;
SET.r.variable = SET.variable ;

SET.r.ss     = length(SET.r.data{1}(1,:)) ;

SET.r.params_to_estimate = ...
    {'alph', 'rho', 'rhow', ...
     'rhoz', 'rhom', 'rhoh', 'rhon', 'rhob', ...
     'sigmaz', 'sigmam', 'sigmah', 'sigman', 'sigmab'} ;
 
SET.r.params_to_estimate_idx = [] ; 
 
for qq=1:length(SET.r.params_to_estimate)
    tmp = find(contains(SET.r.param_names,SET.r.params_to_estimate{qq}),1) ;
    SET.r.params_to_estimate_idx(qq) = tmp ;
end

SET.r.EST.hidx = 1:SET.r.nobs ;
SET.r.EST.hidx_no_obs = [] ;

%% Obtain panel of shocks

xx = SET.r.M_.params(SET.r.params_to_estimate_idx) ;

dyn_in.M_       = SET.r.M_ ;
dyn_in.oo_      = SET.r.oo_ ;
dyn_in.options_ = SET.r.options_ ;
dyn_in.solve    = 1 ;
dyn_in.linearize_around_diff_y = 0 ;

mats = r_resolve_(SET,dyn_in) ;

if isempty(mats) ; out = -10e12 ; return ; end

Pt = mats.mats.Q(1:end-1,1:end-1,:) ;
Dt = mats.mats.G(1:end-1,:,:) ;    

Omega  = SET.r.Omega ;
DOmega = Dt(:,:,1)*Omega*Dt(:,:,1)' ;
SET.r.Sigma = dlyap_doubling(Pt(:,:,1),DOmega);

[~, Ct, Pt, Dt, Qt, Gt] = r_ll(xx, mats, SET, r_data{1}) ;

V  = zeros(SET.r.nobs,SET.r.nobs) ; 
H  = zeros(SET.r.nobs,n_-1) ; 

for ii=1:SET.r.nobs ; H(ii,SET.r.EST.obs_(ii)) = 1 ; end
    
SET.r.ss = length(SET.r.data{1}(1,:)) ;

for ii=1:SET.r.Nstate
    [x_T{ii}, n_T{ii}] = ks_(SET.r, Ct, Pt, Dt, H, V, r_data{ii}, SET.TT) ;
end

%% Change shocks and simulate

SET.nsim = Q_idx(end) ;

datevec = 1999:0.25:2015 ;

Q = Qt(:,:,1) ;
G = Gt(:,:,1) ;

n_  = SET.r.variable.n_ ;
yss = [(eye(n_-1) - Q(1:n_-1,1:n_-1)) \ Q(1:n_-1,end) ; 1] ;

for jj=1:SET.variable.l_
    for ii=1:SET.r.Nstate
        etmat = n_T{ii} ;
        
	    % change shocks here
        % varexo em eh ez en eb;
        tmp=1:SET.variable.l_;
        tmp(jj) = [] ;
        etmat(tmp,:) = 0 ;
        
        ysim      = zeros(n_,SET.nsim) ;
        ysim(:,1) = yss ;
        
        for tt=2:SET.nsim
            ysim(:,tt) = Q*ysim(:,tt-1) + G*etmat(:,tt) ;
        end
        
        simmat{ii}{jj} = ysim ;
    end
end

%% Contributions in Changes

for jj=1:SET.variable.l_
    for ii=1:SET.r.Nstate
        xx(1:SET.nsim,ii,jj)=simmat{ii}{jj}(SET.variable.spendobs_,:)';
        yy(1:SET.nsim,ii)=x_T{ii}(SET.variable.spendobs_,:)';
    end
end

for jj=1:SET.variable.l_
    tmpx = xx(datevec==2007,:,jj) - xx(datevec==2002,:,jj);
	tmpy = yy(datevec==2007,:) - yy(datevec==2002,:);
    tmp = fitlm(tmpy(:),tmpx(:), 'y ~ -1+x1');
    slopes(3,jj)=tmp.Coefficients.Estimate(1);
end

for jj=1:SET.variable.l_
    tmpx = xx(datevec==2010,:,jj) - xx(datevec==2007,:,jj);
	tmpy = yy(datevec==2010,:) - yy(datevec==2007,:);
    tmp=fitlm(tmpy(:),tmpx(:), 'y ~ -1+x1');
    slopes(4,jj)=tmp.Coefficients.Estimate(1);
end

for jj=1:SET.variable.l_
    for ii=1:SET.r.Nstate
        xx(1:SET.nsim,ii,jj)=simmat{ii}{jj}(SET.variable.nobs_,:)';
        yy(1:SET.nsim,ii)=x_T{ii}(SET.variable.nobs_,:)';
    end
end

for jj=1:SET.variable.l_
    tmpx = xx(datevec==2007,:,jj) - xx(datevec==2002,:,jj);
	tmpy = yy(datevec==2007,:) - yy(datevec==2002,:);
    tmp=fitlm(tmpy(:),tmpx(:), 'y ~ -1+x1');
    slopes(1,jj)=tmp.Coefficients.Estimate(1);
end

for jj=1:SET.variable.l_
    tmpx = xx(datevec==2010,:,jj) - xx(datevec==2007,:,jj);
	tmpy = yy(datevec==2010,:) - yy(datevec==2007,:);
    tmp=fitlm(tmpy(:),tmpx(:), 'y ~ -1+x1');
    slopes(2,jj)=tmp.Coefficients.Estimate(1);
end

disp(' ') ;
disp('Contribution of Individual Shocks to Observed Changes, slopes')
latex(slopes,'%.2f','nomath')
