
clear all
clc

warning('Off','all') ;

path(pathdef)

addpath('./output') ;
addpath('./aggregate') ;
addpath('./regional') ;
addpath('./routines') ;
addpath('./routines/solmat') ;
addpath('./../data/aggregate') ;
addpath('./../data/state') ;

SET.experiment = 'baseline' ;

SET.statetrend = 1 ;
SET.popweights = 1 ;
SET.debtinreg  = 0 ;
SET.removetop5states = 0 ;

%% Set up regional matrices and data

r_setup_model

SET.r.param_names = cellstr(dyn_in.M_.param_names) ;

r_gen_set_params(SET,dyn_in)

NYears        = 17 ;
NQuarters     = NYears*4 ;
Q_idx         = 1:4:NQuarters ;
SET.r.TT(Q_idx) = 1 ;
SET.r.Nstate    = 51 ;

r_data = [];

r_load_data

SET.r.nobs = length(r_data{1}(:,1));
SET.r.EST.obs_ = [ ...
    SET.variable.spendobs_
    SET.variable.nobs_
    SET.variable.wobs_
    SET.variable.debtobs_
    SET.variable.pobs_] ;

SET.r.data = r_data ;
SET.r.M_ = M_ ;
SET.r.oo_ = oo_ ;
SET.r.options_ = options_ ;
SET.r.variable = SET.variable ;

SET.r.ss     = length(SET.r.data{1}(1,:)) ;

SET.r.params_to_estimate = ...
    {'alph', 'rho', 'rhow', ...
     'rhoz', 'rhom', 'rhoh', 'rhon', 'rhob', ...
     'sigmaz', 'sigmam', 'sigmah', 'sigman', 'sigmab'} ;
 
SET.r.params_to_estimate_idx = [] ; 
 
for qq=1:length(SET.r.params_to_estimate)
    tmp = find(contains(SET.r.param_names,SET.r.params_to_estimate{qq}),1) ;
    SET.r.params_to_estimate_idx(qq) = tmp ;
end

SET.r.EST.hidx = 1:SET.r.nobs ;
SET.r.EST.hidx_no_obs = [] ;

%% Get structural shocks

xx = SET.r.M_.params(SET.r.params_to_estimate_idx) ;

dyn_in.M_       = SET.r.M_ ;
dyn_in.oo_      = SET.r.oo_ ;
dyn_in.options_ = SET.r.options_ ;
dyn_in.solve    = 1 ;
dyn_in.linearize_around_diff_y = 0 ;

mats = r_resolve_(SET,dyn_in) ;

if isempty(mats) ; out = -10e12 ; return ; end

Pt = mats.mats.Q(1:end-1,1:end-1,:) ;
Dt = mats.mats.G(1:end-1,:,:) ;    

Omega  = SET.r.Omega ;
DOmega = Dt(:,:,1)*Omega*Dt(:,:,1)' ;
SET.r.Sigma = dlyap_doubling(Pt(:,:,1),DOmega);

[~, Ct, Pt, Dt, Qt, Gt] = r_ll(xx, mats, SET, r_data{1}) ;

V  = zeros(SET.r.nobs,SET.r.nobs) ; 
H  = zeros(SET.r.nobs,n_-1) ; 

for ii=1:SET.r.nobs ; H(ii,SET.r.EST.obs_(ii)) = 1 ; end
    
SET.r.ss = length(SET.r.data{1}(1,:)) ;

for ii=1:SET.r.Nstate
    [x_T{ii}, n_T{ii}] = ks_(SET.r, Ct, Pt, Dt, H, V, r_data{ii}, SET.TT) ;
end

%% Change shocks and simulate

SET.nsim = Q_idx(end) ;

datevec = 1999:0.25:2015 ;

Q = Qt(:,:,1) ;
G = Gt(:,:,1) ;

n_  = SET.r.variable.n_ ;
yss = [(eye(n_-1) - Q(1:n_-1,1:n_-1)) \ Q(1:n_-1,end) ; 1] ;

for ii=1:SET.r.Nstate
    ytr      = zeros(n_,SET.nsim) ;
    ytr(:,1) = [x_T{ii}(:,1);1] ;
    
    for tt=2:SET.nsim
        ytr(:,tt) = Q * ytr(:,tt-1) ;
    end
    
    simmattr{ii} = ytr ;
end

for ii=1:SET.r.Nstate
    % change shocks here
    % varexo em eh ez en eb;
    etmat = n_T{ii} ;
    etmat([3 4 5],:) = 0 ;
    
    ysim      = zeros(n_,SET.nsim) ;
    ysim(:,1) = [x_T{ii}(:,1);1] ;
    
    for tt=2:SET.nsim
        ysim(:,tt) = Q * ysim(:,tt-1) + G * etmat(:,tt) ;
    end
    
    simmat_em_eh{ii} = ysim - simmattr{ii} ;
end

for ii=1:SET.r.Nstate
    datasimtr{ii} = simmattr{ii}(SET.r.EST.obs_,1:end) ;
    datasim_em_eh{ii} = simmat_em_eh{ii}(SET.r.EST.obs_,1:end) ;
end

for ii=1:SET.r.Nstate
    tmp0(:,ii) = x_T{ii}(SET.variable.nobs_,Q_idx)' ;
    tmp1(:,ii) = simmat_em_eh{ii}(SET.variable.nobs_,Q_idx)' ;
end

% structure for output csv file:
%  - year, state, five variables from data, five variables from model

mat_out = [] ;
 
for ii=1:SET.r.Nstate
    mat_out = [mat_out ; [1999+(0:(NYears-1))', ii*ones(NYears,1), ...
        x_T{ii}(SET.r.EST.obs_,Q_idx)' simmat_em_eh{ii}(SET.r.EST.obs_,Q_idx)']];
end

mat_out(:,[3 4 5 8 9 10]) = mat_out(:,[3 4 5 8 9 10]) / 100 ;
mat_out(:,[7 12]) = mat_out(:,[7 12]) / 10 ;

eval(['csvwrite(''./output/results_matlab_', SET.experiment, '_em_eh.csv'', mat_out);']);    
eval(['csvwrite(''../stata/figure-6/results_matlab_', SET.experiment, '_em_eh.csv'', mat_out);']);    
