
clear all 
set more off 

insheet year statenum lc_data ln_data lw_data debt_data lq_data lc_model ln_model lw_model debt_model lq_model using "results_matlab_baseline_em_eh.csv", comma 

replace debt_data = debt_data/4   
replace debt_model = debt_model/4 

sort statenum year 
merge statenum using statename
drop _merge  

rename statename state
order state

xtset statenum year 
sort statenum year  

g Dn_bust = (ln_data   - L3.ln_data)        if year == 2010 
g Dc_bust = (lc_data   - L3.lc_data)    	if year == 2010 
g Dq_bust = (lq_data   - L3.lq_data)    	if year == 2010 
g Dw_bust = (lw_data   - L3.lw_data)    	if year == 2010 
g Dd_bust = (F.debt_data - L2.debt_data)  	if year == 2010    /* shift one since stock at 2010 determined a year earlier */

g Dn_boom = (ln_data   - L5.ln_data)       	if year == 2007
g Dc_boom = (lc_data   - L5.lc_data)    	if year == 2007 
g Dq_boom = (lq_data   - L5.lq_data)    	if year == 2007 
g Dw_boom = (lw_data   - L5.lw_data)    	if year == 2007 
g Dd_boom = (F.debt_data - L4.debt_data)  	if year == 2007    /* shift one since stock at 2006 determined a year earlier */

g Dn_bust_model = (ln_model     - L3.ln_model)    if year == 2010 
g Dc_bust_model = (lc_model     - L3.lc_model)    if year == 2010 
g Dq_bust_model = (lq_model     - L3.lq_model)    if year == 2010 
g Dw_bust_model = (lw_model     - L3.lw_model)    if year == 2010 
g Dd_bust_model = (F.debt_model - L2.debt_model)  if year == 2010 

g Dn_boom_model = (ln_model     - L5.ln_model)    if year == 2007
g Dc_boom_model = (lc_model     - L5.lc_model)    if year == 2007 
g Dq_boom_model = (lq_model     - L5.lq_model)    if year == 2007 
g Dw_boom_model = (lw_model     - L5.lw_model)    if year == 2007 
g Dd_boom_model = (F.debt_model - L4.debt_model)  if year == 2007 


twoway (scatter Dn_bust Dd_bust if year == 2010, c(i l) mlabel(state) mlabc(gs0) ms(i i) yti(Log Change in Emp/Pop 07-10) xti(Change HH Debt 07 - 10 (rel. 2001 income)) xlab(-0.5(0.2)0.1) ylabel( ,grid)) ,  saving(ed_bust, replace)   

twoway (scatter Dn_boom Dd_boom if year == 2007, c(i l) mlabel(state) mlabc(gs0) ms(i i) legend(off) yti(Log Change in Emp/Pop 02-07) xti(Change in HH Debt 02 - 07 (rel. 2001 income)) xlab(-.25(0.25)0.5) ylabel( ,grid)) , saving(ed_boom, replace)   

twoway (scatter Dc_bust Dd_bust if year == 2010, c(i l) mlabel(state) mlabc(gs0) ms(i i) legend(off) yti(Log Change in Spending 07-10) xti(Change HH Debt 07 - 10 (rel. 2001 income)) xlab(-0.5(0.2)0.1) ylabel(,grid)) , saving(cd_bust, replace)   

twoway (scatter Dc_boom Dd_boom if year == 2007, c(i l) mlabel(state) mlabc(gs0) ms(i i) legend(off) yti(Log Change in Spending 02-07) xti(Change in HH Debt 02 - 07 (rel. 2001 income)) xlab(-.25(0.25)0.5) ylabel(,grid)) , saving(cd_boom, replace) 

graph combine ed_boom.gph cd_boom.gph ed_bust.gph cd_bust.gph 

*
*
*

foreach stat in "Dq" "Dn" "Dc" "Dw"{

reg `stat'_boom_model `stat'_boom, nocons
mat beta = e(b)
local loc1 = beta[1,1]

local `stat'_R = string(`loc1', "%3.2f")

corr `stat'_boom_model `stat'_boom
local loc1 = r(rho)
local `stat'_corr = string(`loc1', "%3.2f")

}

scatter Dn_boom_model Dn_boom, text(.037 -.0275 "slope = `Dn_R'") text(.027 -.0275 "corr = `Dn_corr'") c(i l) mlabel(state) ylabel(,grid) mlabcolor(black) mlabposition(0) mlabsize(2.2) ms(i i) legend(off) title(A. {&Delta} Employment 02-07, size(3.5)) yti(Model (credit shocks)) xti(Data) xscale(titlegap(*5)) yscale(titlegap(*0)) xsc(r(-0.04 0.04)) ysc(r(-0.04 0.04)) xlabel(-0.04 -0.02 0 0.02 0.04) ylabel(-0.04 -0.02 0 0.02 0.04) || line Dn_boom Dn_boom, sort  lwidth(medthick ..)  saving(temp2, replace)

scatter Dc_boom_model Dc_boom, text(.0925 -.065 "slope = `Dc_R'") text(.0675 -.065 "corr = `Dc_corr'") c(i l) mlabel(state) ylabel(,grid) mlabcolor(black) mlabposition(0) mlabsize(2.2) ms(i i) legend(off) title(B. {&Delta} Consumption 02-07, size(3.5)) yti(Model (credit shocks)) xti(Data) xscale(titlegap(*5)) xsc(r(-0.1 0.1)) yscale(titlegap(*0)) || line Dc_boom Dc_boom, sort  lwidth(medthick ..)  saving(temp3, replace)

foreach stat in "Dq" "Dn" "Dc" "Dw"{

reg `stat'_bust_model `stat'_bust, nocons
mat beta = e(b)
local loc1 = beta[1,1]

local `stat'_R = string(`loc1', "%3.2f")

corr `stat'_bust_model `stat'_bust
local loc1 = r(rho)
local `stat'_corr = string(`loc1', "%3.2f")
}
 
scatter Dn_bust_model Dn_bust, text(.035 -.025 "slope = `Dn_R'") text(.025 -.025 "corr = `Dn_corr'")  c(i l) mlabel(state) ylabel(,grid) mlabcolor(black) mlabposition(0) mlabsize(2.2) ms(i i) legend(off) title(C. {&Delta} Employment 07-10, size(3.5)) yti(Model (credit shocks)) xti(Data) xscale(titlegap(*5)) yscale(titlegap(*0)) || line Dn_bust Dn_bust, sort  lwidth(medthick ..)  saving(temp5, replace) 

scatter Dc_bust_model Dc_bust, text(.055 -.04 "slope = `Dc_R'") text(.04 -.04 "corr = `Dc_corr'") c(i l) mlabel(state) ylabel(,grid) mlabcolor(black) mlabposition(0) mlabsize(2.2) ms(i i) legend(off) title(D. {&Delta} Consumption 07-10, size(3.5)) yti(Model (credit shocks)) xsc(r(-0.06 0.06)) xlabel(-0.06 -0.03 0 0.03 0.06) ysc(r(-0.06 0.06)) ylabel(-0.06 -0.03 0 0.03 0.06) xti(Data) xscale(titlegap(*5))  yscale(titlegap(*0)) || line Dc_bust Dc_bust, sort  lwidth(medthick ..)  saving(temp6, replace) 

graph combine temp2.gph temp3.gph temp5.gph temp6.gph, rows(2) cols(2) ysize(2) xsize(2.25) iscale(*1.0) 
